library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;

entity top_level is

Port (   fpga_0_RS232_RX_pin : in std_logic;
			fpga_0_RS232_TX_pin : out std_logic;
			mpmc_0_SDRAM_Clk_pin : out std_logic;
			mpmc_0_SDRAM_CE_pin : out std_logic;
			mpmc_0_SDRAM_CS_n_pin : out std_logic;
			mpmc_0_SDRAM_RAS_n_pin : out std_logic;
			mpmc_0_SDRAM_CAS_n_pin : out std_logic;
			mpmc_0_SDRAM_WE_n_pin : out std_logic;
			mpmc_0_SDRAM_BankAddr_pin : out std_logic_vector(1 downto 0);
			mpmc_0_SDRAM_Addr_pin : out std_logic_vector(12 downto 0);
			mpmc_0_SDRAM_DQ : inout std_logic_vector(7 downto 0);
			mpmc_0_SDRAM_DM_pin : out std_logic_vector(0 downto 0);
			led					: out std_logic_vector(3 downto 0);
			btn					: in std_logic_vector(1 downto 0);
			clk_test				: out std_logic;
			clk					: in std_logic);
end top_level;

architecture Behavioral of top_level is

signal count: std_logic_vector(23 downto 0);
signal count2: std_logic_vector(2 downto 0);
signal btn_st: std_logic;
signal btn1: std_logic;
signal btn2: std_logic;
signal fade: std_logic;
signal reset_two_btn: std_logic;
signal pwm: std_logic_vector(7 downto 0);

  component system is
    port (
      fpga_0_RS232_RX_pin : in std_logic;
      fpga_0_RS232_TX_pin : out std_logic;
      fpga_0_clk_1_sys_clk_pin : in std_logic;
      fpga_0_rst_1_sys_rst_pin : in std_logic;
      mpmc_0_SDRAM_Clk_pin : out std_logic;
      mpmc_0_SDRAM_CE_pin : out std_logic;
      mpmc_0_SDRAM_CS_n_pin : out std_logic;
      mpmc_0_SDRAM_RAS_n_pin : out std_logic;
      mpmc_0_SDRAM_CAS_n_pin : out std_logic;
      mpmc_0_SDRAM_WE_n_pin : out std_logic;
      mpmc_0_SDRAM_BankAddr_pin : out std_logic_vector(1 downto 0);
      mpmc_0_SDRAM_Addr_pin : out std_logic_vector(12 downto 0);
      mpmc_0_SDRAM_DQ : inout std_logic_vector(7 downto 0);
      mpmc_0_SDRAM_DM_pin : out std_logic_vector(0 downto 0)
    );
  end component;
  
 attribute BOX_TYPE : STRING;
 attribute BOX_TYPE of system : component is "user_black_box";

begin

  system_i : system
    port map (
      fpga_0_RS232_RX_pin => fpga_0_RS232_RX_pin,
      fpga_0_RS232_TX_pin => fpga_0_RS232_TX_pin,
      fpga_0_clk_1_sys_clk_pin => CLK,
      fpga_0_rst_1_sys_rst_pin => reset_two_btn,
      mpmc_0_SDRAM_Clk_pin => mpmc_0_SDRAM_Clk_pin,
      mpmc_0_SDRAM_CE_pin => mpmc_0_SDRAM_CE_pin,
      mpmc_0_SDRAM_CS_n_pin => mpmc_0_SDRAM_CS_n_pin,
      mpmc_0_SDRAM_RAS_n_pin => mpmc_0_SDRAM_RAS_n_pin,
      mpmc_0_SDRAM_CAS_n_pin => mpmc_0_SDRAM_CAS_n_pin,
      mpmc_0_SDRAM_WE_n_pin => mpmc_0_SDRAM_WE_n_pin,
      mpmc_0_SDRAM_BankAddr_pin => mpmc_0_SDRAM_BankAddr_pin,
      mpmc_0_SDRAM_Addr_pin => mpmc_0_SDRAM_Addr_pin,
      mpmc_0_SDRAM_DQ => mpmc_0_SDRAM_DQ,
      mpmc_0_SDRAM_DM_pin => mpmc_0_SDRAM_DM_pin
    );

	LDC_1: LDC
	port map(	D		=> '1',
					G		=> btn1,
					CLR	=> btn2,
					Q		=> btn_st);
	
	count_proc: process(clk)
	begin
		if clk'event and clk = '1' then
			count <= count + '1';
		end if;
	end process;

	count2_proc: process(count(23))
	begin
		if count(23)'event and count(23) = '1' then
		if (btn_st='0')		then count2 <= count2 + '1';
		elsif (btn_st='1')	then count2 <= count2 - '1';
		end if;
		end if;
	end process;

leds: process(count2,fade,pwm)
begin
	if (fade = '0') then
   case count2 is 
      when "000" => led <= "0000";
		when "001" => led <= "0001";
		when "010" => led <= "0011";
		when "011" => led <= "0111";
		when "100" => led <= "1111";
		when "101" => led <= "1110";
		when "110" => led <= "1100";
		when "111" => led <= "1000";
      when others => led <= "0000";
   end case;
	elsif (fade = '1') then
	   case count2 is 
      when "000" => led(0) <= pwm(0); led(1) <= pwm(0); led(2) <= pwm(0); led(3) <= pwm(0);
		when "001" => led(0) <= pwm(1); led(1) <= pwm(1); led(2) <= pwm(1); led(3) <= pwm(1);
		when "010" => led(0) <= pwm(2); led(1) <= pwm(2); led(2) <= pwm(2); led(3) <= pwm(2);
		when "011" => led(0) <= pwm(3); led(1) <= pwm(3); led(2) <= pwm(3); led(3) <= pwm(3);
		when "100" => led(0) <= pwm(4); led(1) <= pwm(4); led(2) <= pwm(4); led(3) <= pwm(4);
		when "101" => led(0) <= pwm(5); led(1) <= pwm(5); led(2) <= pwm(5); led(3) <= pwm(5);
		when "110" => led(0) <= pwm(6); led(1) <= pwm(6); led(2) <= pwm(6); led(3) <= pwm(6);
		when "111" => led(0) <= pwm(7); led(1) <= pwm(7); led(2) <= pwm(7); led(3) <= pwm(7);
      when others => led <= "0000";
   end case;
	end if;
end process;

buttons: process(btn)
	begin
		case btn is 
      when "01" => btn1 <= '1'; btn2 <= '0'; fade <= '0'; reset_two_btn <= '1';
      when "10" => btn1 <= '0'; btn2 <= '1'; fade <= '0'; reset_two_btn <= '1';
		when "00" => btn1 <= '0'; btn2 <= '0'; fade <= '1'; reset_two_btn <= '0';
      when others => btn1 <= '0'; btn2 <= '0'; fade <= '0'; reset_two_btn <= '1';
   end case;
	end process;

pwm(7) <= '1';
pwm(6) <= count(17) or count(16);
pwm(5) <= count(16);
pwm(4) <= count(16) and count (15);
pwm(3) <= count(18) and count (17) and count (16);
pwm(2) <= count(18) and count (17)and count (16)and count (15);
pwm(1) <= count(18) and count (17)and count (16)and count (15)and count (14) and count (13);
pwm(0) <= '0';

clk_test <= count(5);

end Behavioral;